@@ -1,4 +1,4 @@
-/*	$OpenBSD: bgpd.h,v 1.394 2019/10/01 08:57:47 claudio Exp $ */
+/*	$OpenBSD: bgpd.h,v 1.402 2020/04/23 16:13:11 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -75,10 +75,10 @@
 #endif
 #define	CONFFILE			SYSCONFDIR "/bgpd.conf"
 
-#ifndef LOCALSTATEDIR
-#define LOCALSTATEDIR "/var"
+#ifndef RUNSTATEDIR
+#define RUNSTATEDIR "/var/run"
 #endif
-#define	SOCKET_NAME			LOCALSTATEDIR "/run/bgpd.sock"
+#define	SOCKET_NAME			RUNSTATEDIR "/bgpd.sock"
 
 #define	F_BGPD_INSERTED		0x0001
 #define	F_KERNEL		0x0002
@@ -375,7 +375,8 @@ struct capabilities {
 
 struct peer_config {
 	struct bgpd_addr	 remote_addr;
-	struct bgpd_addr	 local_addr;
+	struct bgpd_addr	 local_addr_v4;
+	struct bgpd_addr	 local_addr_v6;
 	struct peer_auth	 auth;
 	struct capabilities	 capabilities;
 	char			 group[PEER_DESCR_LEN];
@@ -389,10 +390,12 @@ struct peer_config {
 	u_int32_t		 remote_as;
 	u_int32_t		 local_as;
 	u_int32_t		 max_prefix;
+	u_int32_t		 max_out_prefix;
 	enum export_type	 export_type;
 	enum enforce_as		 enforce_as;
 	enum enforce_as		 enforce_local_as;
 	u_int16_t		 max_prefix_restart;
+	u_int16_t		 max_out_prefix_restart;
 	u_int16_t		 holdtime;
 	u_int16_t		 min_holdtime;
 	u_int16_t		 local_short_as;
@@ -584,7 +587,9 @@ enum suberr_cease {
 	ERR_CEASE_CONN_REJECT,
 	ERR_CEASE_OTHER_CHANGE,
 	ERR_CEASE_COLLISION,
-	ERR_CEASE_RSRC_EXHAUST
+	ERR_CEASE_RSRC_EXHAUST,
+	ERR_CEASE_HARD_RESET,
+	ERR_CEASE_MAX_SENT_PREFIX
 };
 
 struct kroute_node;
@@ -666,7 +671,8 @@ struct kif {
 };
 
 struct session_up {
-	struct bgpd_addr	local_addr;
+	struct bgpd_addr	local_v4_addr;
+	struct bgpd_addr	local_v6_addr;
 	struct bgpd_addr	remote_addr;
 	struct capabilities	capa;
 	u_int32_t		remote_bgpid;
@@ -721,17 +727,16 @@ struct ctl_show_rib {
 	struct bgpd_addr	prefix;
 	struct bgpd_addr	remote_addr;
 	char			descr[PEER_DESCR_LEN];
-	time_t			lastchange;
+	time_t			age;
 	u_int32_t		remote_id;
 	u_int32_t		local_pref;
 	u_int32_t		med;
 	u_int32_t		weight;
 	u_int32_t		flags;
-	u_int16_t		aspath_len;
 	u_int8_t		prefixlen;
 	u_int8_t		origin;
 	u_int8_t		validation_state;
-	/* plus a aspath_len bytes long aspath */
+	/* plus an aspath */
 };
 
 enum as_spec {
@@ -1188,8 +1193,6 @@ void		free_prefixtree(struct prefixset_tree *);
 void		filterlist_free(struct filter_head *);
 int		host(const char *, struct bgpd_addr *, u_int8_t *);
 u_int32_t	get_bgpid(void);
-void		copy_filterset(struct filter_set_head *,
-		    struct filter_set_head *);
 void		expand_networks(struct bgpd_config *);
 int		prefixset_cmp(struct prefixset_item *, struct prefixset_item *);
 RB_PROTOTYPE(prefixset_tree, prefixset_item, entry, prefixset_cmp);
@@ -1267,10 +1270,10 @@ int	pftable_addr_remove(struct pftable_msg *);
 int	pftable_commit(void);
 
 /* rde_filter.c */
-void		 filterset_free(struct filter_set_head *);
-int		 filterset_cmp(struct filter_set *, struct filter_set *);
-void		 filterset_move(struct filter_set_head *,
-		    struct filter_set_head *);
+void	filterset_free(struct filter_set_head *);
+int	filterset_cmp(struct filter_set *, struct filter_set *);
+void	filterset_move(struct filter_set_head *, struct filter_set_head *);
+void	filterset_copy(struct filter_set_head *, struct filter_set_head *);
 const char	*filterset_name(enum action_types);
 
 /* rde_sets.c */
@@ -1302,6 +1305,9 @@ int	trie_roa_check(struct trie_head *, struct bgpd_addr *, u_int8_t,
 void	trie_dump(struct trie_head *);
 int	trie_equal(struct trie_head *, struct trie_head *);
 
+/* timer.c */
+time_t			 getmonotime(void);
+
 /* util.c */
 const char	*log_addr(const struct bgpd_addr *);
 const char	*log_in6addr(const struct in6_addr *);
@@ -1440,14 +1446,16 @@ static const char * const suberr_update_names[] = {
 
 static const char * const suberr_cease_names[] = {
 	"none",
-	"max-prefix exceeded",
+	"received max-prefix exceeded",
 	"administratively down",
 	"peer unconfigured",
 	"administrative reset",
 	"connection rejected",
 	"other config change",
 	"collision",
-	"resource exhaustion"
+	"resource exhaustion",
+	"hard reset",
+	"sent max-prefix exceeded"
 };
 
 static const char * const ctl_res_strerror[] = {
