@@ -1,4 +1,4 @@
-/*	$OpenBSD: session.c,v 1.395 2019/10/02 08:57:00 claudio Exp $ */
+/*	$OpenBSD: session.c,v 1.400 2020/04/23 16:13:11 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004, 2005 Henning Brauer <henning@openbsd.org>
@@ -33,6 +33,7 @@
 #include <err.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <ifaddrs.h>
 #include <poll.h>
 #include <pwd.h>
 #include <signal.h>
@@ -252,7 +253,7 @@ session_main(int debug, int verbose)
 				/* cloned peer that idled out? */
 				if (p->template && (p->state == STATE_IDLE ||
 				    p->state == STATE_ACTIVE) &&
-				    time(NULL) - p->stats.last_updown >=
+				    getmonotime() - p->stats.last_updown >=
 				    INTERVAL_HOLD_CLONED)
 					p->reconf_action = RECONF_DELETE;
 
@@ -610,6 +611,8 @@ bgp_fsm(struct peer *peer, enum session_events event)
 
 			peer->stats.last_sent_errcode = 0;
 			peer->stats.last_sent_suberr = 0;
+			peer->stats.last_rcvd_errcode = 0;
+			peer->stats.last_rcvd_suberr = 0;
 
 			if (!peer->depend_ok)
 				timer_stop(peer, Timer_ConnectRetry);
@@ -1036,6 +1039,7 @@ int
 session_connect(struct peer *peer)
 {
 	struct sockaddr		*sa;
+	struct bgpd_addr	*bind_addr = NULL;
 	socklen_t		 sa_len;
 
 	/*
@@ -1063,8 +1067,16 @@ session_connect(struct peer *peer)
 	tcp_md5_set(peer->fd, peer);
 	peer->wbuf.fd = peer->fd;
 
-	/* if update source is set we need to bind() */
-	if ((sa = addr2sa(&peer->conf.local_addr, 0, &sa_len)) != NULL) {
+	/* if local-address is set we need to bind() */
+	switch (peer->conf.remote_addr.aid) {
+	case AID_INET:
+		bind_addr = &peer->conf.local_addr_v4;
+		break;
+	case AID_INET6:
+		bind_addr = &peer->conf.local_addr_v6;
+		break;
+	}
+	if (bind_addr && (sa = addr2sa(bind_addr, 0, &sa_len)) != NULL) {
 		if (bind(peer->fd, sa, sa_len) == -1) {
 			log_peer_warn(&peer->conf, "session_connect bind");
 			bgp_fsm(peer, EVNT_CON_OPENFAIL);
@@ -1194,6 +1206,69 @@ session_setup_socket(struct peer *p)
 	return (0);
 }
 
+/* compare two sockaddrs by converting them into bgpd_addr */
+static int
+sa_cmp(struct sockaddr *a, struct sockaddr *b)
+{
+	struct bgpd_addr ba, bb;
+
+	sa2addr(a, &ba, NULL);
+	sa2addr(b, &bb, NULL);
+
+	return (memcmp(&ba, &bb, sizeof(ba)) == 0);
+}
+
+static void
+get_alternate_addr(struct sockaddr *sa, struct bgpd_addr *alt)
+{
+	struct ifaddrs	*ifap, *ifa, *match;
+
+	if (getifaddrs(&ifap) == -1)
+		fatal("getifaddrs");
+
+	for (match = ifap; match != NULL; match = match->ifa_next)
+		if (sa_cmp(sa, match->ifa_addr) == 0)
+			break;
+
+	if (match == NULL) {
+		log_warnx("%s: local address not found", __func__);
+		return;
+	}
+
+	switch (sa->sa_family) {
+	case AF_INET6:
+		for (ifa = ifap; ifa != NULL; ifa = ifa->ifa_next) {
+			if (ifa->ifa_addr->sa_family == AF_INET &&
+			    strcmp(ifa->ifa_name, match->ifa_name) == 0) {
+				sa2addr(ifa->ifa_addr, alt, NULL);
+				break;
+			}
+		}
+		break;
+	case AF_INET:
+		for (ifa = ifap; ifa != NULL; ifa = ifa->ifa_next) {
+			struct sockaddr_in6 *s =
+			    (struct sockaddr_in6 *)ifa->ifa_addr;
+			if (ifa->ifa_addr->sa_family == AF_INET6 &&
+			    strcmp(ifa->ifa_name, match->ifa_name) == 0) {
+				/* only accept global scope addresses */
+				if (IN6_IS_ADDR_LINKLOCAL(&s->sin6_addr) ||
+				    IN6_IS_ADDR_SITELOCAL(&s->sin6_addr))
+					continue;
+				sa2addr(ifa->ifa_addr, alt, NULL);
+				break;
+			}
+		}
+		break;
+	default:
+		log_warnx("%s: unsupported address family %d", __func__,
+		    sa->sa_family);
+		break;
+	}
+
+	freeifaddrs(ifap);
+}
+
 void
 session_tcp_established(struct peer *peer)
 {
@@ -1204,6 +1279,7 @@ session_tcp_established(struct peer *peer)
 	if (getsockname(peer->fd, (struct sockaddr *)&ss, &len) == -1)
 		log_warn("getsockname");
 	sa2addr((struct sockaddr *)&ss, &peer->local, &peer->local_port);
+	get_alternate_addr((struct sockaddr *)&ss, &peer->local_alt);
 	len = sizeof(ss);
 	if (getpeername(peer->fd, (struct sockaddr *)&ss, &len) == -1)
 		log_warn("getpeername");
@@ -1706,6 +1782,7 @@ session_dispatch_msg(struct pollfd *pfd, struct peer *p)
 			bgp_fsm(p, EVNT_CON_FATAL);
 			return (1);
 		}
+		p->stats.last_write = getmonotime();
 		if (p->throttled && p->wbuf.queued < SESS_MSG_LOW_MARK) {
 			if (imsg_rde(IMSG_XON, p->conf.id, NULL, 0) == -1)
 				log_peer_warn(&p->conf, "imsg_compose XON");
@@ -1731,7 +1808,7 @@ session_dispatch_msg(struct pollfd *pfd, struct peer *p)
 		}
 
 		p->rbuf->wpos += n;
-		p->stats.last_read = time(NULL);
+		p->stats.last_read = getmonotime();
 		return (1);
 	}
 	return (0);
@@ -2193,6 +2270,8 @@ parse_notification(struct peer *peer)
 
 	log_notification(peer, errcode, subcode, p, datalen, "received");
 	peer->errcnt++;
+	peer->stats.last_rcvd_errcode = errcode;
+	peer->stats.last_rcvd_suberr = subcode;
 
 	if (errcode == ERR_OPEN && subcode == ERR_OPEN_CAPA) {
 		if (datalen == 0) {	/* zebra likes to send those.. humbug */
@@ -2511,6 +2590,7 @@ session_dispatch_imsg(struct imsgbuf *ibuf, int idx, u_int *listener_cnt)
 	struct kif		*kif;
 	u_char			*data;
 	int			 n, fd, depend_ok, restricted;
+	u_int16_t		 t;
 	u_int8_t		 aid, errcode, subcode;
 
 	while (ibuf) {
@@ -2795,10 +2875,15 @@ session_dispatch_imsg(struct imsgbuf *ibuf, int idx, u_int *listener_cnt)
 			case ERR_CEASE:
 				switch (subcode) {
 				case ERR_CEASE_MAX_PREFIX:
+				case ERR_CEASE_MAX_SENT_PREFIX:
+					t = p->conf.max_out_prefix_restart;
+					if (subcode == ERR_CEASE_MAX_PREFIX)
+						t = p->conf.max_prefix_restart;
+
 					bgp_fsm(p, EVNT_STOP);
-					if (p->conf.max_prefix_restart)
-						timer_set(p, Timer_IdleHold, 60 *
-						    p->conf.max_prefix_restart);
+					if (t)
+						timer_set(p, Timer_IdleHold,
+						    60 * t);
 					break;
 				default:
 					bgp_fsm(p, EVNT_CON_FATAL);
@@ -3053,7 +3138,7 @@ void
 session_down(struct peer *peer)
 {
 	bzero(&peer->capa.neg, sizeof(peer->capa.neg));
-	peer->stats.last_updown = time(NULL);
+	peer->stats.last_updown = getmonotime();
 	/*
 	 * session_down is called in the exit code path so check
 	 * if the RDE is still around, if not there is no need to
@@ -3074,13 +3159,19 @@ session_up(struct peer *p)
 	    &p->conf, sizeof(p->conf)) == -1)
 		fatalx("imsg_compose error");
 
-	sup.local_addr = p->local;
+	if (p->local.aid == AID_INET) {
+		sup.local_v4_addr = p->local;
+		sup.local_v6_addr = p->local_alt;
+	} else {
+		sup.local_v6_addr = p->local;
+		sup.local_v4_addr = p->local_alt;
+	}
 	sup.remote_addr = p->remote;
 
 	sup.remote_bgpid = p->remote_bgpid;
 	sup.short_as = p->short_as;
 	memcpy(&sup.capa, &p->capa.neg, sizeof(sup.capa));
-	p->stats.last_updown = time(NULL);
+	p->stats.last_updown = getmonotime();
 	if (imsg_rde(IMSG_SESSION_UP, p->conf.id, &sup, sizeof(sup)) == -1)
 		fatalx("imsg_compose error");
 }
